#include <stdio.h>
#include "serial.h"

void Serial::Initialize(void)
{
	DCB dcbSerialParams = {0};
	COMMTIMEOUTS timeouts = {0};

	hSerial = CreateFile("COM1",
						GENERIC_READ | GENERIC_WRITE,
						0,
						0,
						OPEN_EXISTING,
						FILE_ATTRIBUTE_NORMAL,
						0);
						
	if(hSerial==INVALID_HANDLE_VALUE)
	{
		if(GetLastError()==ERROR_FILE_NOT_FOUND)
			printf("Serial port does not exist.\n");
		else
			printf("Serial port error.\n");
	}
	else
	{
		dcbSerialParams.BaudRate=CBR_115200;
		dcbSerialParams.ByteSize=8;
		dcbSerialParams.StopBits=ONESTOPBIT;
		dcbSerialParams.Parity=NOPARITY;
		if(!SetCommState(hSerial, &dcbSerialParams))
			printf("Error setting serial port state.\n");
			
		timeouts.ReadIntervalTimeout=50;
		timeouts.ReadTotalTimeoutConstant=50;
		timeouts.ReadTotalTimeoutMultiplier=10;
		timeouts.WriteTotalTimeoutConstant=50;
		timeouts.WriteTotalTimeoutMultiplier=10;
		if(!SetCommTimeouts(hSerial, &timeouts))
			printf("Error setting serial timeouts.\n");
	}
}

void Serial::readACK(void)
{
	DWORD dwBytesRead = 0;
	char szBuffer[101];

	ReadFile(hSerial, szBuffer, sizeof(szBuffer)-1, &dwBytesRead, NULL);
	szBuffer[dwBytesRead] = '\0';
}

void Serial::grabImage(char *filename)
{
	DWORD dwBytesRead = 0;
	char szBuffer[101];

	strcpy(szBuffer, "SF\r");
	WriteFile(hSerial, szBuffer, strlen(szBuffer), &dwBytesRead, NULL);
	ReadFile(hSerial, cmu_image, sizeof(cmu_image)-1, &dwBytesRead, NULL);
	cmu_image[dwBytesRead] = '\0';
	
	width = (int)cmu_image[5];
	height = (int)cmu_image[6];
	printf("Frame Grab Complete\n");
}

void Serial::grabImage2(char *filename)
{
	DWORD dwBytesRead = 0;
	char szBuffer[101];

	strcpy(szBuffer, "SF 0\r");
	WriteFile(hSerial, szBuffer, strlen(szBuffer), &dwBytesRead, NULL);
	ReadFile(hSerial, cmu_image2, sizeof(cmu_image2)-1, &dwBytesRead, NULL);
	cmu_image2[dwBytesRead] = '\0';
	
	width = (int)cmu_image2[5];
	height = (int)cmu_image2[6];
	printf("Frame Grab Complete\n");
}

void Serial::sendServo(int servo_val, int direction)
{
	DWORD dwBytesRead = 0;
	char command[255];

	if ((direction == SERVO_X_DIRECTION) && (servo_x_val != servo_val))
	{
		sprintf(command, "SV 1 %d\r", servo_val);
		WriteFile(hSerial, command, strlen(command), &dwBytesRead, NULL);
		readACK();
		servo_x_val = servo_val;
	}

	if ((direction == SERVO_Y_DIRECTION) && (servo_y_val != servo_val))
	{
		sprintf(command, "SV 0 %d\r", servo_val);
		WriteFile(hSerial, command, strlen(command), &dwBytesRead, NULL);
		readACK();
		servo_y_val = servo_val;
	}
}

